
/////////////////////////////////////////////////////////////////////
// OCB (the Object Clustering Benchmark)
// Version 1.0 OCB/O2, janvier 1999
// Fromantin Christophe
// LIMOS, Blaise Pascal University (Clermont-Ferrand II), France
/////////////////////////////////////////////////////////////////////
// ocb.cc: Benchmark execution protocol
/////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "o2lib_CC.hxx"
#include "OQL_CC.hxx"
#include "o2template_CC.hxx"
#include "OCB_O2.h"
#include "OCB_Util_O2.cc"
#include "OCB_Object.hxx"
#include "OCB_Class.hxx"
#include "OCB_Object.cc"
#include "OCB_Class.cc"
#include "OCB_Classes_O2.hxx"
#include "OCB_Methods_O2.cc"

/////////////////////////////////////////////////////////////////////
// Specific functions prototypes 
/////////////////////////////////////////////////////////////////////

void GetParameters(int narg,char *carg[]);
void DisplayParameters();

/////////////////////////////////////////////////////////////////////
// Main program
/////////////////////////////////////////////////////////////////////

main(int narg,  char*carg[])
 {

  ////////////////////////////////////////
  // Declarations & initializations     //
  ////////////////////////////////////////

  long IO,n1,n2,t1,t2,d1,d2;            // Performance measures
  int i,k;

  printf("\n*** OCB v1.0 ***\n");


  // Dynamic OCB parameters input in line or in interactive mode

  GetParameters(narg,carg);
  DisplayParameters();

  // Random seed initialization 

   Randomize(RSEED);


  //////////////////////////////////////// 
  // Database generation                //
  ////////////////////////////////////////
 
  OCB_db_O2 *Base_O2_OCB;

  Base_O2_OCB = new OCB_db_O2(narg,carg);
  Base_O2_OCB->Create_Database_O2();
  Base_O2_OCB->Open_Database_O2();

  IO=GetIO();time(&t1);
  Base_O2_OCB->Create_Class_Object_O2();
  time(&t2);

  printf("\nDatabase generation: %d s, %d I/Os\n",t2-t1,GetIO()-IO);

  // Deletion of object db / database is closed
  delete Base_O2_OCB;


  ////////////////////////////////////////
  // Usage                              //
  ////////////////////////////////////////

  Base_O2_OCB = new OCB_db_O2(narg,carg);
  Base_O2_OCB->Open_Database_O2();
  OCB_Object::Restart_Object(1);

  for (k=0; k<CLIENTN; k++) {           // For each client
                                        // Transactions
    Base_O2_OCB->ResetStats();
    IO=GetIO();time(&t1);	
    for (i=0; i<HOTN; i++) {
       Base_O2_OCB->RunTransaction(NOSTATS);}
    time(&t2);
    printf("\nDatabase usage: %d s, %d I/Os\n",t2-t1,GetIO()-IO);
    printf("\nMean response time: %f ms\n",
      1000.*(t2-t1)/(Base_O2_OCB->NSet+Base_O2_OCB->NSim+Base_O2_OCB->NHie+Base_O2_OCB->NSto));
    Base_O2_OCB->DisplayStats();
    }

  // Database content display
    printf("\n");

  // Deletion of object db / database is closed
  delete Base_O2_OCB;

}


/////////////////////////////////////////////////////////////////////
// Dynamic OCB parameters input in line or in interactive mode
/////////////////////////////////////////////////////////////////////

//void GetParameters(int narg, const char **carg) {
void GetParameters(int narg,char *carg[]) {

// Dynamic parameters: NC, NO, COLDN, HOTN, P*, RSEED

  if(narg>1) { // In line
    NC=atoi(carg[1]);
    NO=atoi(carg[2]);
    COLDN=atoi(carg[3]);
    HOTN=atoi(carg[4]);
    PSET=atof(carg[5]);
    PSIMPLE=atof(carg[6]);
    PHIER=atof(carg[7]);
    PSTOCH=atof(carg[8]);
    RSEED=atoi(carg[9]);
    }
  else { // Interactive mode
    printf("\nDynamic parameters input:\n");
    printf("\nNC=");scanf("%d",&NC);
    printf("NO=");scanf("%d",&NO);
    printf("COLDN=");scanf("%d",&COLDN);
    printf("HOTN=");scanf("%d",&HOTN);
    printf("PSET=");scanf("%f",&PSET);
    printf("PSIMPLE=");scanf("%f",&PSIMPLE);
    printf("PHIER=");scanf("%f",&PHIER);
    printf("PSTOCH=");scanf("%f",&PSTOCH);
    printf("RSEED=");scanf("%d",&RSEED);
    }

  // Updates

  SUPREF=NO;
  SUPCLASS=NC;
}

/////////////////////////////////////////////////////////////////////
// OCB parameters display
/////////////////////////////////////////////////////////////////////

void DisplayParameters() {
  printf("\nOCB parameters:");
  printf("\t\tNC=%d\t\t|\tSETDEPH=%d\n",NC,SETDEPTH);
  printf("--------------\t\tMAXNREF=%d\t|\tSIMDEPH=%d\n",MAXNREF,SIMDEPTH);
  printf("\t\t\tBASESIZE=%d\t|\tHIEDEPH=%d\n",BASESIZE,HIEDEPTH);
  printf("\t\t\tNO=%d\t|\tSTODEPH=%d\n",NO,STODEPTH);
  printf("\t\t\tNREFT=%d\t\t|\tCOLDN=%d\n",NREFT,COLDN);
  printf("\t\t\tINFCLASS=%d\t|\tHOTN=%d\n",INFCLASS,HOTN);
  printf("\t\t\tSUPCLASS=%d\t|\tTHINK=%d\n",SUPCLASS,THINK);
  printf("\t\t\tINFREF=%d\t|\tPSET=%f\n",INFREF,PSET);
  printf("\t\t\tSUPREF=%d\t|\tPSIMPLE=%f\n",SUPREF,PSIMPLE);
  printf("\t\t\tDIST1=%d\t\t|\tPHIER=%f\n",DIST1,PHIER);
  printf("\t\t\tDIST2=%d\t\t|\tPSTOCH=%f\n",DIST2,PSTOCH);
  printf("\t\t\tDIST3=%d\t\t|\tDIST5=%d\n",DIST3,DIST5);
  printf("\t\t\tDIST4=%d\t\t|\tCLIENTN=%d\n",DIST4,CLIENTN);
  printf("\t\t\t\t\t|\tRSEED=%d\n",RSEED);
}

